@@ -18,7 +18,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include "config.h"
+#include "config.h" // For GLOM_ENABLE_SQLITE, GLOM_ENABLE_CLIENT_ONLY
 
 #include "frame_glom.h"
 #include "application.h"
@@ -29,7 +29,10 @@
 #include <glom/libglom/connectionpool.h>
 #include <glom/libglom/connectionpool_backends/postgres_central.h>
 #include <glom/libglom/connectionpool_backends/postgres_self.h>
-#include <glom/libglom/connectionpool_backends/sqlite.h>
+
+#ifdef GLOM_ENABLE_SQLITE
+# include <glom/libglom/connectionpool_backends/sqlite.h>
+#endif
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
 #include "mode_design/users/dialog_groups_list.h"
@@ -329,107 +332,99 @@ void Frame_Glom::show_table_refresh()
   show_table(m_table_name);
 }
 
-void Frame_Glom::show_table(const Glib::ustring& table_name, const Gnome::Gda::Value& primary_key_value_for_details)
+void Frame_Glom::show_table_allow_empty(const Glib::ustring& table_name, const Gnome::Gda::Value& primary_key_value_for_details)
 {
   App_Glom* pApp = dynamic_cast<App_Glom*>(get_app_window());
 
   //This can take quite a long time, so we show the busy cursor while it's working:
   Bakery::BusyCursor busy_cursor(pApp);
 
-  //Check that there is a table to show:
-  if(table_name.empty())
-  {
-    alert_no_table();
-  }
-  else
-  {
-    //Choose a default mode, if necessary:
-    if(m_Mode == MODE_None)
-      set_mode(m_Mode);
+  //Choose a default mode, if necessary:
+  if(m_Mode == MODE_None)
+    set_mode(m_Mode);
 
-    //Show the table:
-    m_table_name = table_name;
-    Glib::ustring strMode;
+  //Show the table:
+  m_table_name = table_name;
+  Glib::ustring strMode;
 #ifndef GLOM_ENABLE_CLIENT_ONLY
-    //Update the document with any new information in the database if necessary (though the database _should never have changed information)
-    update_table_in_document_from_database();
+  //Update the document with any new information in the database if necessary (though the database _should never have changed information)
+  update_table_in_document_from_database();
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
-    //Update user-level dependent UI:
-    if(pApp)
-      on_userlevel_changed(pApp->get_userlevel());
+  //Update user-level dependent UI:
+  if(pApp)
+    on_userlevel_changed(pApp->get_userlevel());
 
-    switch(m_Mode)
+  switch(m_Mode)
+  {
+    case(MODE_Data):
     {
-      case(MODE_Data):
+      strMode = _("Data");
+      FoundSet found_set;
+
+      //Start with the last-used found set (sort order and where clause)
+      //for this layout:
+      //(This would be ignored anyway if a details primary key is specified.)
+      Document_Glom* document = get_document(); 
+      if(document)
+        found_set = document->get_criteria_current(m_table_name);
+
+      //Make sure that this is set:
+      found_set.m_table_name = m_table_name;
+
+      //If there is no saved sort clause, 
+      //then sort by the ID, just so we sort by something, so that the order is predictable:
+      if(found_set.m_sort_clause.empty())
       {
-        strMode = _("Data");
-        FoundSet found_set;
+        sharedptr<Field> field_primary_key = get_field_primary_key_for_table(m_table_name);
+        if(field_primary_key)
+      {
+          sharedptr<LayoutItem_Field> layout_item_sort = sharedptr<LayoutItem_Field>::create();
+          layout_item_sort->set_full_field_details(field_primary_key);
 
-        //Start with the last-used found set (sort order and where clause)
-        //for this layout:
-        //(This would be ignored anyway if a details primary key is specified.)
-        Document_Glom* document = get_document(); 
-        if(document)
-          found_set = document->get_criteria_current(m_table_name);
+          found_set.m_sort_clause.clear();
 
-        //Make sure that this is set:
-        found_set.m_table_name = m_table_name;
-
-        //If there is no saved sort clause, 
-        //then sort by the ID, just so we sort by something, so that the order is predictable:
-        if(found_set.m_sort_clause.empty())
-        {
-          sharedptr<Field> field_primary_key = get_field_primary_key_for_table(m_table_name);
-          if(field_primary_key)
-          {
-            sharedptr<LayoutItem_Field> layout_item_sort = sharedptr<LayoutItem_Field>::create();
-            layout_item_sort->set_full_field_details(field_primary_key);
-
-            found_set.m_sort_clause.clear();
-
-            //Avoid the sort clause if the found set will include too many records, 
-            //because that would be too slow.
-            //The user can explicitly request a sort later, by clicking on a column header.
-            //TODO_Performance: This causes an almost-duplicate COUNT query (we do it in the treemodel too), but it's not that slow. 
-            sharedptr<LayoutItem_Field> layout_item_temp = sharedptr<LayoutItem_Field>::create();
-            layout_item_temp->set_full_field_details(field_primary_key);
-            type_vecLayoutFields layout_fields;
-            layout_fields.push_back(layout_item_temp);
-            const Glib::ustring sql_query_without_sort = Utils::build_sql_select_with_where_clause(found_set.m_table_name, layout_fields, found_set.m_where_clause, found_set.m_extra_join, type_sort_clause(), found_set.m_extra_group_by);
-            const int count = Base_DB::count_rows_returned_by(sql_query_without_sort);
-            if(count < 10000) //Arbitrary large number.
-              found_set.m_sort_clause.push_back( type_pair_sort_field(layout_item_sort, true /* ascending */) );
-          }
+          //Avoid the sort clause if the found set will include too many records, 
+          //because that would be too slow.
+          //The user can explicitly request a sort later, by clicking on a column header.
+          //TODO_Performance: This causes an almost-duplicate COUNT query (we do it in the treemodel too), but it's not that slow. 
+          sharedptr<LayoutItem_Field> layout_item_temp = sharedptr<LayoutItem_Field>::create();
+          layout_item_temp->set_full_field_details(field_primary_key);
+          type_vecLayoutFields layout_fields;
+          layout_fields.push_back(layout_item_temp);
+          const Glib::ustring sql_query_without_sort = Utils::build_sql_select_with_where_clause(found_set.m_table_name, layout_fields, found_set.m_where_clause, found_set.m_extra_join, type_sort_clause(), found_set.m_extra_group_by);
+          const int count = Base_DB::count_rows_returned_by(sql_query_without_sort);
+          if(count < 10000) //Arbitrary large number.
+            found_set.m_sort_clause.push_back( type_pair_sort_field(layout_item_sort, true /* ascending */) );
         }
-
-        //Show the wanted records in the notebook, showing details for a particular record if wanted:
-        m_Notebook_Data.init_db_details(found_set, primary_key_value_for_details);
-        set_mode_widget(m_Notebook_Data);
-
-        //Show how many records were found:
-        update_records_count();
-
-        break;
-      }
-      case(MODE_Find):
-      {
-        strMode = _("Find");
-        m_Notebook_Find.init_db_details(m_table_name, get_document()->get_active_layout_platform());
-        set_mode_widget(m_Notebook_Find);
-        break;
-      }
-      default:
-      {
-        std::cout << "Frame_Glom::on_box_tables_selected(): Unexpected mode" << std::endl;
-        strMode = _("Unknown");
-        break;
       }
+
+      //Show the wanted records in the notebook, showing details for a particular record if wanted:
+      m_Notebook_Data.init_db_details(found_set, primary_key_value_for_details);
+      set_mode_widget(m_Notebook_Data);
+
+      //Show how many records were found:
+      update_records_count();
+
+      break;
+    }
+    case(MODE_Find):
+    {
+      strMode = _("Find");
+      m_Notebook_Find.init_db_details(m_table_name, get_document()->get_active_layout_platform());
+      set_mode_widget(m_Notebook_Find);
+      break;
+    }
+    default:
+    {
+      std::cout << "Frame_Glom::on_box_tables_selected(): Unexpected mode" << std::endl;
+      strMode = _("Unknown");
+      break;
     }
-
-    m_pLabel_Mode->set_text(strMode);
   }
 
+  m_pLabel_Mode->set_text(strMode);
+
   show_table_title();
 
   //List the reports and print layouts in the menus:
@@ -439,6 +434,24 @@ void Frame_Glom::show_table(const Glib::ustring& table_name, const Gnome::Gda::V
   //show_all();
 }
 
+void Frame_Glom::show_table(const Glib::ustring& table_name, const Gnome::Gda::Value& primary_key_value_for_details)
+{
+  //Check that there is a table to show:
+  if(table_name.empty())
+  {
+    alert_no_table();
+  }
+  else
+  {
+    show_table_allow_empty(table_name, primary_key_value_for_details);
+  }
+}
+
+void Frame_Glom::show_no_table()
+{
+  show_table_allow_empty(Glib::ustring());
+}
+
 #ifndef GLOM_ENABLE_CLIENT_ONLY
 void Frame_Glom::on_menu_userlevel_Developer(const Glib::RefPtr<Gtk::RadioAction>& action, const Glib::RefPtr<Gtk::RadioAction>& operator_action)
 {
@@ -564,7 +577,7 @@ void Frame_Glom::on_menu_file_export()
     return;
 
   dialog.get_layout_groups(mapGroupSequence);
-  std::cout << "DEBUG 0: mapGroupSequence.size()=" << mapGroupSequence.size() << std::endl;
+  //std::cout << "DEBUG 0: mapGroupSequence.size()=" << mapGroupSequence.size() << std::endl;
 
   //const int index_primary_key = fieldsSequence.size() - 1;
 
@@ -580,6 +593,57 @@ void Frame_Glom::on_menu_file_export()
   export_data_to_stream(the_stream, found_set, mapGroupSequence);
 }
 
+//TODO: Reduce copy/pasting in these export_data_to_*() methods:
+void Frame_Glom::export_data_to_vector(Document_Glom::type_example_rows& the_vector, const FoundSet& found_set, const Document_Glom::type_list_layout_groups& sequence)
+{
+  type_vecLayoutFields fieldsSequence = get_table_fields_to_show_for_sequence(found_set.m_table_name, sequence);
+
+  if(fieldsSequence.empty())
+  {
+    std::cerr << "Glom: Frame_Glom::export_data_to_string(): No fields in sequence." << std::endl;
+    return;
+  }
+
+  const Glib::ustring query = Utils::build_sql_select_with_where_clause(found_set.m_table_name, fieldsSequence, found_set.m_where_clause, found_set.m_extra_join, found_set.m_sort_clause, found_set.m_extra_group_by);
+
+  //TODO: Lock the database (prevent changes) during export.
+  Glib::RefPtr<Gnome::Gda::DataModel> result = query_execute_select(query);
+
+  guint rows_count = 0;
+  if(result)
+    rows_count = result->get_n_rows();
+
+  if(rows_count)
+  {
+    const guint columns_count = result->get_n_columns();
+
+    for(guint row_index = 0; row_index < rows_count; ++row_index)
+    {
+        Document_Glom::type_row_data row_data;
+
+        for(guint col_index = 0; col_index < columns_count; ++col_index)
+        {
+          const Gnome::Gda::Value value = result->get_value_at(col_index, row_index);
+
+          sharedptr<LayoutItem_Field> layout_item = fieldsSequence[col_index];
+          //if(layout_item->m_field.get_glom_type() != Field::TYPE_IMAGE) //This is too much data.
+          //{
+
+            //Output data in canonical SQL format, ignoring the user's locale, and ignoring the layout formatting:
+            row_data.push_back(value);  //TODO_Performance: reserve the size.
+
+            //if(layout_item->m_field.get_glom_type() == Field::TYPE_IMAGE) //This is too much data.
+            //{
+             //std::cout << "  field name=" << layout_item->get_name() << ", value=" << layout_item->m_field.sql(value) << std::endl;
+            //}
+        }
+
+        //std::cout << " row_string=" << row_string << std::endl;
+        the_vector.push_back(row_data); //TODO_Performance: Reserve the size.
+    }
+  }
+}
+
 void Frame_Glom::export_data_to_string(Glib::ustring& the_string, const FoundSet& found_set, const Document_Glom::type_list_layout_groups& sequence)
 {
   type_vecLayoutFields fieldsSequence = get_table_fields_to_show_for_sequence(found_set.m_table_name, sequence);
@@ -593,7 +657,7 @@ void Frame_Glom::export_data_to_string(Glib::ustring& the_string, const FoundSet
   const Glib::ustring query = Utils::build_sql_select_with_where_clause(found_set.m_table_name, fieldsSequence, found_set.m_where_clause, found_set.m_extra_join, found_set.m_sort_clause, found_set.m_extra_group_by);
 
   //TODO: Lock the database (prevent changes) during export.
-  Glib::RefPtr<Gnome::Gda::DataModel> result = query_execute_select(query, get_app_window());
+  Glib::RefPtr<Gnome::Gda::DataModel> result = query_execute_select(query);
 
   guint rows_count = 0;
   if(result)
@@ -618,7 +682,7 @@ void Frame_Glom::export_data_to_string(Glib::ustring& the_string, const FoundSet
               row_string += ",";
 
             //Output data in canonical SQL format, ignoring the user's locale, and ignoring the layout formatting:
-            row_string += layout_item->get_full_field_details()->sql(value, Field::SQL_FORMAT_POSTGRES);
+            row_string += layout_item->get_full_field_details()->to_file_format(value);
 
             //if(layout_item->m_field.get_glom_type() == Field::TYPE_IMAGE) //This is too much data.
             //{
@@ -645,7 +709,7 @@ void Frame_Glom::export_data_to_stream(std::ostream& the_stream, const FoundSet&
   const Glib::ustring query = Utils::build_sql_select_with_where_clause(found_set.m_table_name, fieldsSequence, found_set.m_where_clause, found_set.m_extra_join, found_set.m_sort_clause, found_set.m_extra_group_by);
  
   //TODO: Lock the database (prevent changes) during export.
-  Glib::RefPtr<Gnome::Gda::DataModel> result = query_execute_select(query, get_app_window());
+  Glib::RefPtr<Gnome::Gda::DataModel> result = query_execute_select(query);
 
   guint rows_count = 0;
   if(result)
@@ -670,7 +734,7 @@ void Frame_Glom::export_data_to_stream(std::ostream& the_stream, const FoundSet&
               row_string += ",";
 
             //Output data in canonical SQL format, ignoring the user's locale, and ignoring the layout formatting:
-            row_string += layout_item->get_full_field_details()->sql(value, Field::SQL_FORMAT_POSTGRES);
+            row_string += layout_item->get_full_field_details()->to_file_format(value);
 
             if(layout_item->get_glom_type() == Field::TYPE_IMAGE) //This is too much data.
             {
@@ -699,6 +763,14 @@ void Frame_Glom::on_menu_file_import()
     Gtk::FileChooserDialog file_chooser(*get_app_window(), _("Choose a CSV file to open"), Gtk::FILE_CHOOSER_ACTION_OPEN);
     file_chooser.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
     file_chooser.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_ACCEPT);
+    Gtk::FileFilter filter_csv;
+    filter_csv.set_name(_("CSV files"));
+    filter_csv.add_mime_type("text/csv");
+    file_chooser.add_filter(filter_csv);
+    Gtk::FileFilter filter_any;
+    filter_any.set_name(_("All files"));
+    filter_any.add_pattern("*");
+    file_chooser.add_filter(filter_any);
 
     if(file_chooser.run() == Gtk::RESPONSE_ACCEPT)
     {
@@ -1317,6 +1389,15 @@ void Frame_Glom::do_menu_developer_fields(Gtk::Window& parent, const Glib::ustri
     add_view(m_pDialog_Fields);
   }
 
+  // Some database backends (SQLite) require the table to change no longer
+  // being in use when changing the records, so clear the database usage
+  // here. We reshow everything in on_developer_dialog_hide anyway.
+  show_no_table();
+
+  // Remember the old table name so that we re-show the previous table as
+  // soon as the dialog has been closed.
+  m_table_name = table_name;
+
   m_pDialog_Fields->set_transient_for(parent);
   m_pDialog_Fields->init_db_details(table_name);
   m_pDialog_Fields->show();
@@ -1562,30 +1643,34 @@ namespace
     ConnectionPool* connection_pool = ConnectionPool::get_instance();
     switch(document->get_hosting_mode())
     {
+#ifndef GLOM_ENABLE_CLIENT_ONLY
     case Document_Glom::POSTGRES_SELF_HOSTED:
       {
         ConnectionPoolBackends::PostgresSelfHosted* backend = new ConnectionPoolBackends::PostgresSelfHosted;
         backend->set_self_hosting_data_uri(document->get_connection_self_hosted_directory_uri());
-        connection_pool->set_backend(std::auto_ptr<ConnectionPoolBackend>(backend));
+        connection_pool->set_backend(std::auto_ptr<ConnectionPool::Backend>(backend));
       }
       break;
+#endif //GLOM_ENABLE_CLIENT_ONLY
     case Document_Glom::POSTGRES_CENTRAL_HOSTED:
       {
         ConnectionPoolBackends::PostgresCentralHosted* backend = new ConnectionPoolBackends::PostgresCentralHosted;
         backend->set_host(document->get_connection_server());
         backend->set_port(document->get_connection_port());
         backend->set_try_other_ports(document->get_connection_try_other_ports());
-        connection_pool->set_backend(std::auto_ptr<ConnectionPoolBackend>(backend));
+        connection_pool->set_backend(std::auto_ptr<ConnectionPool::Backend>(backend));
       }
 
       break;
+#ifdef GLOM_ENABLE_SQLITE
     case Document_Glom::SQLITE_HOSTED:
       {
         ConnectionPoolBackends::Sqlite* backend = new ConnectionPoolBackends::Sqlite;
         backend->set_database_directory_uri(document->get_connection_self_hosted_directory_uri());
-        connection_pool->set_backend(std::auto_ptr<ConnectionPoolBackend>(backend));
+        connection_pool->set_backend(std::auto_ptr<ConnectionPool::Backend>(backend));
       }
       break;
+#endif // GLOM_ENABLE_SQLITE
     default:
       g_assert_not_reached();
       break;
@@ -1789,7 +1874,7 @@ bool Frame_Glom::connection_request_password_and_choose_new_database_name()
         // Remember host if the document is not self hosted
         if(document->get_hosting_mode() == Document_Glom::POSTGRES_CENTRAL_HOSTED)
         {
-          ConnectionPoolBackend* backend = connection_pool->get_backend();
+          ConnectionPool::Backend* backend = connection_pool->get_backend();
           ConnectionPoolBackends::PostgresCentralHosted* central = dynamic_cast<ConnectionPoolBackends::PostgresCentralHosted*>(backend);
           g_assert(central != NULL);
 
@@ -1844,11 +1929,15 @@ bool Frame_Glom::connection_request_password_and_attempt(const Glib::ustring kno
     // not support authentication. I'd prefer to get that information from
     // libgda, but gda_connection_supports_feature() requires a GdaConnection
     // which we don't have at this point.
-    if(document->get_hosting_mode() != Document_Glom::SQLITE_HOSTED &&
-       known_username.empty() && known_password.empty())
+#ifdef GLOM_ENABLE_SQLITE
+    if(document->get_hosting_mode() != Document_Glom::SQLITE_HOSTED)
+#endif
     {
-       response = Glom::Utils::dialog_run_with_help(m_pDialogConnection, "dialog_connection");
-       m_pDialogConnection->hide();
+      if(known_username.empty() && known_password.empty())
+      {
+        response = Glom::Utils::dialog_run_with_help(m_pDialogConnection, "dialog_connection");
+        m_pDialogConnection->hide();
+      }
     }
 
     if(response == Gtk::RESPONSE_OK)
