@@ -22,7 +22,6 @@
 #include <glom/libglom/data_structure/glomconversions.h>
 #include <glom/libglom/glade_utils.h>
 #include <glom/reports/report_builder.h>
-#include <glom/utility_widgets/db_adddel/glom_db_treemodel.h>
 #include "dialog_layout_list.h"
 #include <glom/glom_privs.h>
 #include <bakery/App/App_Gtk.h> //For util_bold_message().
@@ -578,20 +577,10 @@ void Box_Data_List::get_record_counts(gulong& total, gulong& found) const
   found = 0;
 
   Glib::RefPtr<Gtk::TreeModel> refModel = m_AddDel.get_model();
-  //if(!refModel)
-  //  std::cerr << "Box_Data_List::get_record_counts(): (this=" << typeid(this).name() << ") The tree model was NULL." << std::endl;
-
   Glib::RefPtr<DbTreeModel> refModelDerived = Glib::RefPtr<DbTreeModel>::cast_dynamic(refModel);
   
   if(refModelDerived)
     refModelDerived->get_record_counts(total, found);
-
-  /*
-  else
-  {
-    std::cerr << "Box_Data_List::get_record_counts(): (this=" << typeid(this).name() << ") The tree model was not a DbTreeModel. It was a " << typeid(refModel.operator->()).name() << std::endl;
-  }
-  */
 }
 
 void Box_Data_List::on_adddel_user_sort_clause_changed()
@@ -607,6 +596,11 @@ void Box_Data_List::on_adddel_user_sort_clause_changed()
     document->set_criteria_current(m_table_name, m_found_set);
 }
 
+Gtk::TreeModel::iterator Box_Data_List::get_row_selected()
+{
+  return m_AddDel.get_item_selected();
+}
+
 #ifndef GLOM_ENABLE_CLIENT_ONLY
 //overridden, so we can change the column widths, so they are all visible:
 void Box_Data_List::on_dialog_layout_hide()
