@@ -130,7 +130,7 @@ void Dialog_UsersList::on_button_user_remove()
         if(!user.empty())
         {
           Glib::ustring strQuery = "ALTER GROUP \"" + m_combo_group->get_active_text() + "\" DROP USER \"" + user + "\"";
-          const bool test = query_execute(strQuery, this);
+          const bool test = query_execute(strQuery);
           if(!test)
             std::cerr << "Dialog_UsersList::on_button_user_remove(): ALTER GROUP failed." << std::endl;
 
@@ -169,7 +169,7 @@ void Dialog_UsersList::on_button_user_delete()
           if(response == Gtk::RESPONSE_OK)
           {
             const Glib::ustring strQuery = "DROP USER \"" + user + "\"";
-            const bool test = query_execute(strQuery, this);
+            const bool test = query_execute(strQuery);
             if(!test)
               std::cerr << "Dialog_UsersList::on_button_user_delete(): DROP USER failed" << std::endl;
 
@@ -215,7 +215,7 @@ void Dialog_UsersList::on_button_user_add()
   {
     //Add it to the group:
     const Glib::ustring strQuery = "ALTER GROUP \"" + m_combo_group->get_active_text() + "\" ADD USER \"" + user + "\"";
-    const bool test = query_execute(strQuery, this);
+    const bool test = query_execute(strQuery);
     if(!test)
       std::cerr << "Dialog_UsersList::on_button_user_add(): ALTER GROUP failed." << std::endl;
 
@@ -225,7 +225,7 @@ void Dialog_UsersList::on_button_user_add()
     for(Document_Glom::type_listTableInfo::const_iterator iter = table_list.begin(); iter != table_list.end(); ++iter)
     {
       const Glib::ustring strQuery = "REVOKE ALL PRIVILEGES ON \"" + (*iter)->get_name() + "\" FROM \"" + user + "\"";
-      const bool test = query_execute(strQuery, this);
+      const bool test = query_execute(strQuery);
       if(!test)
         std::cerr << "Dialog_UsersList::on_button_user_add(): REVOKE failed." << std::endl;
     }
@@ -281,13 +281,13 @@ void Dialog_UsersList::on_button_user_new()
   {
     //Create the user:
     Glib::ustring strQuery = "CREATE USER \"" + user + "\" PASSWORD '" + password + "'" ; //TODO: Escape the password.
-    bool test = query_execute(strQuery, this);
+    bool test = query_execute(strQuery);
     if(!test)
        std::cerr << "Dialog_UsersList::on_button_user_new(): CREATE USER failed." << std::endl;
 
     //Add it to the group:
     strQuery = "ALTER GROUP \"" + m_combo_group->get_active_text() + "\" ADD USER \"" + user + "\"";
-    test = query_execute(strQuery, this);
+    test = query_execute(strQuery);
     if(!test)
        std::cerr << "Dialog_UsersList::on_button_user_new(): ALTER GROUP failed." << std::endl;
 
@@ -365,7 +365,7 @@ void Dialog_UsersList::on_button_user_edit()
       if(!user.empty() && !password.empty())
       {
         const Glib::ustring strQuery = "ALTER USER \"" + user + "\" PASSWORD '" + password + "'" ; //TODO: Escape the password.
-        const bool test = query_execute(strQuery, this);
+        const bool test = query_execute(strQuery);
         if(!test)
           std::cerr << "Dialog_UsersList::on_button_user_edit(): ALTER USER failed." << std::endl;
 
