@@ -195,11 +195,27 @@ DataWidget::DataWidget(const sharedptr<LayoutItem_Field>& field, const Glib::ust
     apply_formatting(*m_child, field->get_formatting_used());
     
     bool child_added = false; //Don't use an extra container unless necessary.
-    const bool field_used_in_relationship_to_one = document->get_field_used_in_relationship_to_one(table_name, field->get_name());
+
+    //Check whether the field controls a relationship,
+    //meaning it identifies a record in another table.
+    const bool field_used_in_relationship_to_one = document->get_field_used_in_relationship_to_one(table_name, field);
+    //std::cout << "DEBUG: table_name=" << table_name << ", table_used=" << field->get_table_used(table_name) << ", field=" << field->get_name() << ", field_used_in_relationship_to_one=" << field_used_in_relationship_to_one << std::endl;
+
+    //Check whether the field identifies a record in another table 
+    //just because it is a primary key in that table:
+    bool field_is_related_primary_key = false;
+    if(document)
+      field->set_full_field_details( document->get_field(field->get_table_used(table_name), field->get_name()) ); //Otherwise get_primary_key() returns false always.
+    sharedptr<const Field> field_info = field->get_full_field_details();
+    field_is_related_primary_key = 
+      field->get_has_relationship_name() && 
+      field_info && field_info->get_primary_key();
+    //std::cout <<   "DEBUG: field->get_has_relationship_name()=" << field->get_has_relationship_name() << ", field_info->get_primary_key()=" <<  field_info->get_primary_key() << ", field_is_related_primary_key=" << field_is_related_primary_key << std::endl;
+
 
     Gtk::HBox* hbox_parent = 0; //Only used if there are extra widgets.
 
-    const bool with_extra_widgets = field_used_in_relationship_to_one || (glom_type == Field::TYPE_DATE);
+    const bool with_extra_widgets = field_used_in_relationship_to_one || field_is_related_primary_key || (glom_type == Field::TYPE_DATE);
     if(with_extra_widgets)
     {
       hbox_parent = Gtk::manage( new Gtk::HBox() ); //We put the child (and any extra stuff) in this:
@@ -221,18 +237,24 @@ DataWidget::DataWidget(const sharedptr<LayoutItem_Field>& field, const Glib::ust
       button_date->signal_clicked().connect(sigc::mem_fun(*this, &DataWidget::on_button_choose_date));
     }
 
-    if(field_used_in_relationship_to_one && hbox_parent)
+    if((field_used_in_relationship_to_one || field_is_related_primary_key) && hbox_parent)
     {
-      //Add buttons for related record navigation:
+      //Add a button for related record navigation:
       Gtk::Button* button_go_to_details = Gtk::manage(new Gtk::Button(Gtk::Stock::OPEN));
       button_go_to_details->set_tooltip_text(_("Open the record identified by this ID, in the other table."));
       hbox_parent->pack_start(*button_go_to_details);
       button_go_to_details->signal_clicked().connect(sigc::mem_fun(*this, &DataWidget::on_button_open_details));
 
-      Gtk::Button* button_select = Gtk::manage(new Gtk::Button(Gtk::Stock::FIND));
-      button_select->set_tooltip_text(_("Enter search criteria to identify records in the other table, to choose an ID for this field."));
-      hbox_parent->pack_start(*button_select);
-      button_select->signal_clicked().connect(sigc::mem_fun(*this, &DataWidget::on_button_select_id));
+      //Add a button to make it easier to choose an ID for this field.
+      //Don't add this for simple related primary key fields, because they 
+      //can generally not be edited via another table's layout.
+      if(field_used_in_relationship_to_one)
+      {
+        Gtk::Button* button_select = Gtk::manage(new Gtk::Button(Gtk::Stock::FIND));
+        button_select->set_tooltip_text(_("Enter search criteria to identify records in the other table, to choose an ID for this field."));
+        hbox_parent->pack_start(*button_select);
+        button_select->signal_clicked().connect(sigc::mem_fun(*this, &DataWidget::on_button_select_id));
+      }
     }
 
     if(!child_added)
@@ -356,7 +378,7 @@ int DataWidget::get_suitable_width(const sharedptr<const LayoutItem_Field>& fiel
     }
     case(Field::TYPE_TIME):
     {
-      Gnome::Gda::Time time = {0, 0, 0, 0};
+      Gnome::Gda::Time time = {0, 0, 0, 0, 0};
       time.hour = 24;
       time.minute = 59;
       time.second = 59;
@@ -793,7 +815,7 @@ void DataWidget::on_button_choose_date()
   }
 }
 
-void DataWidget::on_self_style_changed(const Glib::RefPtr<Gtk::Style>& style)
+void DataWidget::on_self_style_changed(const Glib::RefPtr<Gtk::Style>& /* style */)
 {
   sharedptr<LayoutItem_Field> layoutField = sharedptr<LayoutItem_Field>::cast_dynamic(get_layout_item());
   set_child_size_by_field(layoutField);
@@ -838,20 +860,20 @@ bool DataWidget::offer_related_record_id_find(Gnome::Gda::Value& chosen_id)
 
     //Discover the related table, in the relationship that uses this ID field:
     Glib::ustring related_table_name;
-    sharedptr<LayoutItem_Field> layoutField = sharedptr<LayoutItem_Field>::cast_dynamic(get_layout_item());
+    sharedptr<const LayoutItem_Field> layoutField = sharedptr<LayoutItem_Field>::cast_dynamic(get_layout_item());
     if(layoutField)
     {
-      sharedptr<Relationship> relationship = get_document()->get_field_used_in_relationship_to_one(m_table_name, layoutField->get_name());
+      sharedptr<const Relationship> relationship = get_document()->get_field_used_in_relationship_to_one(m_table_name, layoutField);
       if(relationship)
         related_table_name = relationship->get_to_table();
     }
     else
       g_warning("get_layout_item() was not a LayoutItem_Field");
 
-    dialog->init_db_details(related_table_name);
+    dialog->init_db_details(related_table_name, get_document()->get_active_layout_platform());
 
 
-    int response = dialog->run();
+    const int response = dialog->run();
     dialog->hide();
     if(response == Gtk::RESPONSE_OK)
     {
