@@ -832,7 +832,11 @@ void DbAddDel::construct_specified_columns()
   {
     //std::cout << "debug: DbAddDel::construct_specified_columns(): showing hint model: m_find_mode=" << m_find_mode << std::endl;
 
-    show_hint_model();
+    m_refListStore.clear();
+    if(m_table_name.empty())
+      m_TreeView.set_model(m_refListStore); // clear old model from treeview
+    else
+      show_hint_model();
     return;
   }
 
@@ -2247,8 +2251,8 @@ void DbAddDel::user_changed(const Gtk::TreeModel::iterator& row, guint col)
       const bool bTest = set_field_value_in_database(field_in_record, row, field_value, false /* don't use current calculations */, window);
 
       //Glib::ustring strQuery = "UPDATE \"" + table_name + "\"";
-      //strQuery += " SET " +  /* table_name + "." + postgres does not seem to like the table name here */ strFieldName + " = " + field.sql(field_value);
-      //strQuery += " WHERE " + table_name + "." + primary_key_field.get_name() + " = " + primary_key_field.sql(primary_key_value);
+      //strQuery += " SET " +  /* table_name + "." + postgres does not seem to like the table name here */ strFieldName + " = " + field.sql(field_value, connection);
+      //strQuery += " WHERE " + table_name + "." + primary_key_field.get_name() + " = " + primary_key_field.sql(primary_key_value, connection);
       //bool bTest = query_execute(strQuery);
       if(!bTest)
       {
